# -*- coding: utf-8 -*-

"""
Created by Rodrigo Fuentealba
Please, read the file "/LICENSE.md" for licensing information.
"""

__author__="Rodrigo Fuentealba <rfuentealbac.83@gmail.com>"
__version__="1.0.6"
__copyright__="Copyright (c) Rodrigo Fuentealba <rfuentealbac.83@gmail.com>"
__license__="MIT"

import os
import sys
import mimetypes
import mimetools
import StringIO
import quopri
import base64
from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.MIMEImage import MIMEImage
from email.header import Header
from email.utils import parseaddr, formataddr
from pypremailer import Premailer


class message (object):
    """
    This class represents an e-mail message. Contains basic methods and
    attributes to represent an e-mail.
    """

    def __init__(self):
        """
        Begin writing a MIME message with the following structure:
        
            *
            |
            |_ MIMEMultipart("related")
               |_ MIMEMultipart("alternative")
               |  |_ MIMEText -> text/plain
               |  |_ MIMEText -> text/html
               |
               |_ MIMEText
               |  |_ ** Attached text files **
               |
               |_ MIMEImage
               |  |_ ** Attached images **
               |
               |_ MIMEBase
                  |_ ** Other attached files **

        Shown documents belong to a multipart section named alternative
        that is the one the mail user agents display; if something does
        not belong there (i.e. is not displayed) are related documents,
        and belong to the document root (but can be referenced as links
        from the text/html part).
        """
        self.__params__ = {
            "sender": None,
            "from_name": False,
            "from_mail": None,
            "to_name": False,
            "to_mail": None,
            "subject": None,
            "bounce_to": None,
            "errors_to": None,
            "reply_to_name": False,
            "reply_to_mail": None,
            "headers": dict(),
        }

        self.__message__ = MIMEMultipart("related")
        self.__message_text__ = MIMEMultipart("alternative")
        self.__message__.attach(self.__message_text__)

    def utf8ize(self, text):
        """
        Whenever I need a string in Python, I need it encoded as UTF-8.
        """
        if type(text) == type(u""):
            return text.encode("utf-8")
        else:
            return unicode(text, "utf-8", "strict").encode("utf-8")

    def set_string_header(self, hdr):
        """
        Some e-mail headers are strings. Mostly names.
        """
        hdr = self.utf8ize(hdr)
        hdr = Header(hdr, "utf-8")
        return str(hdr)

    def set_email_header(self, hdr):
        hdr = self.utf8ize(hdr)
        try:
            return hdr.encode('ascii')
        except Exception, e:
            raise Exception("Header must be in ASCII format")

    def set_sender(self, mail):
        """
        Validate the "sender" header.
        """
        self.__params__["sender"] = self.utf8ize(mail)

    def set_from_name(self, name):
        """
        Validate the "from name" header.
        Encode it as a variable.
        """
        self.__params__["from_name"] = self.set_string_header(name)

    def set_from_mail(self, mail):
        """
        Validate the "from mail" header.
        Encode it as a variable.
        """
        self.__params__["from_mail"] = self.set_email_header(mail)

    def set_to_name(self, name):
        """
        Validate the "to name" header.
        Encode it as a variable.
        """
        self.__params__["to_name"] = self.set_string_header(name)

    def set_to_mail(self, mail):
        """
        Validate the "to mail" header.
        Encode it as a variable.
        """
        self.__params__["to_mail"] = self.set_email_header(mail)

    def set_subject(self, subject):
        """
        Validate the "subject" header.
        Encode it as a variable.
        """
        self.__params__["subject"] = self.set_string_header(subject)

    def set_reply_to_name(self, name):
        """
        Validate the "reply to name" header.
        Encode it as a variable.
        """
        self.__params__["reply_to_name"] = self.set_string_header(name)

    def set_reply_to_mail(self, mail):
        """
        Validate the "reply to mail" header.
        Encode it as a variable.
        """
        self.__params__["reply_to_mail"] = self.set_email_header(mail)

    def set_bounce_to(self, mail):
        """
        Validate the "bounce to" header.
        Encode it as a variable.
        """
        self.__params__["bounce_to"] = self.set_email_header(mail)

    def set_errors_to(self, mail):
        """
        Validate the "errors to" header.
        Encode it as a variable.
        """
        self.__params__["errors_to"] = self.set_email_header(mail)

    def set_reply_to_mail(self, mail):
        """
        Validate the "reply to mail" header.
        Encode it as a variable.
        """
        self.__params__["reply_to_mail"] = self.set_email_header(mail)

    def add_header(self, index, value):
        """
        Generate an extra header if needed.
        """
        self.__params__["headers"][self.utf8ize(index)] = self.utf8ize(value)

    def add_part(self, part, mimetype="plain"):
        """
        Add multipart alternatives.
        """
        part = MIMEText(part, mimetype)
        if mimetype == "html":
            orig = part.get_payload()
            data = quopri.encodestring(orig)
            part.set_payload(data)
            del part['Content-Transfer-Encoding']
            part['Content-Transfer-Encoding'] = "quoted-printable"
        self.__message_text__.attach(part)

    def add_attachment(self, attached):
        """
        Add attachments to the message.
        
        Detects the mimetype of the file and assigns a MIME type. Then,
        adds it as the related message.
        """
        if not os.path.isfile(attached):
            raise IOError("Attachment does not exist or it is not a file")

        filetype, encoding = mimetypes.guess_type(attached)
        
        if filetype is None or encoding is not None:
            filetype = 'application/octet-stream'

        maintype, subtype = filetype.split("/", 1)
        MIMEClass = None
        
        if maintype == "text":
            MIMEClass = MIMEText
        elif maintype == "image":
            MIMEClass = MIMEImage
        elif maintyoe == "audio":
            MIMEClass = MIMEAudio
        
        fp = open(attached, "rb")
        if MIMEClass is not None:
            msg = MIMEClass(fp.read(), _subtype = subtype)
        else:
            msg = MIMEBase(maintype, subtype)
            msg.set_payload(fp.read())
            encoders.encode_base64(msg)
        fp.close()
        
        msg.add_header("Content-Disposition", "attachment", filename=attached)
        self.__message__.attach(msg)

    def add_image(self, name, content, mime=None):
        """
        Add images inline. It requires the content of the image and the
        reference name.
        
        In the HTML part, the image tag must be written as follows:
        
            <img src="cid:refname" alt="ref" />
        
        If that does not happen, the image will be shown as an attached
        file.
        """
        if mime == None:
            try:
                filetype, encoding = mimetypes.guess_type(attached)
                part = MIMEImage(content, filetype)
            except Exception, e:
                raise Exception("Cannot find MIME type for the image.")
        else:
            part = MIMEImage(content, mime)
        
        part.add_header("Content-ID", "<%s>" % name)
        self.__message__.attach(part)

    def sender(self):
        """
        Returning the sender for use with mailmanager.server.
        """
        return self.__params__["sender"]

    def recipient(self):
        """
        Returning the recipient for use with mailmanager.server.
        """
        return self.__params__["to_mail"]

    def as_string(self):
        """
        Checks the required fields and completes other fields with safe
        values if not present. The result can be sent via SMTP.
        """
        if self.__params__["sender"] == None:
            raise Exception("Can't send without a sender address")
        
        if self.__params__["to_mail"] == None:
            raise Exception("Can't send without a recipient address")
        
        if self.__params__["subject"] == None:
            self.set_subject("No subject")

        for i in ("from_mail", "reply_to_mail"):
            if self.__params__[i] == None:
                self.__params__[i] = self.__params__["sender"]

        del self.__message__["Sender"]
        self.__message__["Sender"] = str(formataddr((
            self.__params__["from_name"],
            self.__params__["from_mail"],
        )))

        del self.__message__["From"]
        self.__message__["From"] = str(formataddr((
            self.__params__["from_name"],
            self.__params__["from_mail"],
        )))
        
        del self.__message__["To"]
        self.__message__["To"] = str(formataddr((
            self.__params__["to_name"],
            self.__params__["to_mail"],
        )))
        
        del self.__message__["Subject"]
        self.__message__["Subject"] = self.__params__["subject"]

        del self.__message__["Reply-To"]
        self.__message__["Reply-To"] = str(formataddr((
            self.__params__["reply_to_name"],
            self.__params__["reply_to_mail"],
        )))

        if "Bounce-To" in self.__message__:
            del self.__message__["Bounce-To"]
        if self.__params__["bounce_to"] is not None:
            self.__message__["Bounce-To"] = str(formataddr((
                False,
                self.__params__["bounce_to"]
            )))
        
        if "Errors-To" in self.__message__:
            del self.__message__["Errors-To"]
        if self.__params__["errors_to"] is not None:
            self.__message__["Errors-To"] = str(formataddr((
                False,
                self.__params__["errors_to"]
            )))
        
        del self.__message__["MIME-Version"]
        self.__message__["MIME-Version"] = "1.0"

        del self.__message__["Priority"]
        self.__message__["Priority"] = "Normal"
        
        for hdr in self.__params__["headers"]:
            del self.__message__[hdr]
            self.__message__[hdr] = str(self.__params__["headers"][hdr])

        return self.__message__.as_string()
