# -*- coding: utf-8 -*-

"""
Created by Rodrigo Fuentealba
Please, read the file "/LICENSE.md" for licensing information.
"""

__author__="Rodrigo Fuentealba <rfuentealbac.83@gmail.com>"
__version__="1.0.6"
__copyright__="Copyright (c) Rodrigo Fuentealba <rfuentealbac.83@gmail.com>"
__license__="MIT"

import smtplib
import time


class server (object):
    """
    Handles a connection to a SMTP server.
    Requires some parameters to be passed through constructors.
    """

    def __init__(self):
        """
        Configure default values for the class.
        """
        self.__params__ = {
            "host": "localhost",
            "port": 25,
            "ssl": False,
            "username": None,
            "password": None,
            "retries": 3,
        }

        self.__connected__ = False

    def set_host(self, host='localhost'):
        """
        Performs a check to see if the host is a string. Encouraged its
        use instead of assigning the parameters directly, it prevents a
        failure while connecting to the server.
        """
        if type(host) in (type('str'), type(u"")):
            self.__params__['host'] = host
        else:
            raise Exception("Expected hostname to be a string.")

    def set_port(self, port=25):
        """
        Performs a check to see if the port is an integer. Encouraged
        its usage instead of assigning parameters directly, because it
        prevents a failure while connecting to the server.
        """
        error = True

        if type(port) in (type(0), type(1.0)):
            self.__params__['port'] = port
            error = False

        if type(port) == type(u""):
            try:
                port = unicode(port)
            except Exception:
                pass

        if type(port) == type('str'):
            try:
                self.__params__['port'] = int(port)
            except Exception:
                pass

        if error == True:
            raise Exception("Expected port number to be a string.")

    def set_ssl(self, ssl):
        """
        Performs a check to see if the SSL usage is declared as boolean
        or converts it accordingly.
        """
        error = True

        if type(ssl) == type(True):
            self.__params__['ssl'] = ssl
            error = False
        elif type(ssl) == type(1):
            if ssl == 1:
                self.__params__['ssl'] == True
                error = False
            elif ssl == 0:
                self.__params__['ssl'] == False
                error = False

        if error == True:
            raise Exception("Expected SSL parameter to be a boolean.")

    def set_username(self, username):
        """
        The same check for host, but now for the username. See above.
        """
        if type(username) in (type('str'), type(u"")):
            self.__params__['username'] = username
        else:
            raise Exception("Expected username to be a string.")

    def set_password(self, password):
        """
        The same check for host, but now for the password. See above.
        """
        if type(password) in (type('str'), type(u"")):
            self.__params__['password'] = password
        else:
            raise Exception("Expected password to be a string.")

    def set_retries(self, retries=3):
        """
        If the client disconnects from the server, we try to reconnect,
        and if it reaches a number of retries configured here, raises a
        SMTPException.
        """
        error = True

        if type(retries) in (type(0), type(1.0)):
            self.__params__['retries'] = retries
            error = False

        if type(retries) == type(u""):
            try:
                retries = unicode(retries)
            except Exception:
                pass

        if type(retries) == type('str'):
            try:
                self.__params__['retries'] = int(retries)
            except Exception:
                pass

        if error == True:
            raise Exception("Expected number of retries to be a string.")

    def connect(self):
        """
        Encapsulates the logic for connecting to the SMTP server. If an
        exception is raised, then retry the connection before raising a
        new one to the global invoker.
        """
        retries = 0
        while self.__connected__ == False and \
            self.__params__['retries'] > retries:
            try:
                self.__server__ = smtplib.SMTP(
                    self.__params__['host'],
                    self.__params__['port'],
                )
                self.__server__.ehlo()
                
                if self.__params__['ssl'] == True:
                    self.__server__.starttls()
                    self.__server__.ehlo()
                
                if self.__params__['username'] is not None and \
                    self.__params__['password'] is not None:
                    self.__server__.login(
                        self.__params__['username'],
                        self.__params__['password'],
                    )
                self.__connected__ = True
            except smtplib.SMTPException, e:
                retries = retries + 1
                if retries < self.__params__['retries']:
                    time.sleep(retries * 5)
            except Exception, e:
                retries = self.__params__['retries']

        if self.__connected__ == False:
            raise Exception(
                "Could not connect to server.",
                500,
                e
            )
        return retries

    def sendmail(self, from_address, to_address, message):
        """
        Tries to send the message. It performs a check to see if there
        is already a connection established. If not, it creates it and
        then sends the message.
        """
        if self.__connected__ == False:
            self.connect()

        try:
            self.__server__.sendmail(from_address, to_address, message)
        except Exception, e:
            raise Exception(
                "Could not send e-mail.",
                404,
                e
            )

    def disconnect(self):
        """
        Tries to unload the connection. If it is already unloaded there
        is nothing to do.
        """
        try:
            self.__server__.quit()
            self.__connected__ = False
        except smtplib.SMTPException:
            pass
